# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'shortcut_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(726, 500)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        Dialog.setMinimumSize(QtCore.QSize(600, 300))
        Dialog.setMaximumSize(QtCore.QSize(1600, 500))
        font = QtGui.QFont()
        font.setKerning(True)
        Dialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setMinimumSize(QtCore.QSize(0, 60))
        self.widget.setMaximumSize(QtCore.QSize(16777215, 60))
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(20)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setTextFormat(QtCore.Qt.AutoText)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout.addWidget(self.widget)
        self.widget_2 = QtWidgets.QWidget(Dialog)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.widget_2.setFont(font)
        self.widget_2.setObjectName("widget_2")
        self.gridLayout = QtWidgets.QGridLayout(self.widget_2)
        self.gridLayout.setObjectName("gridLayout")
        self.label_51 = QtWidgets.QLabel(self.widget_2)
        self.label_51.setMaximumSize(QtCore.QSize(25, 25))
        self.label_51.setText("")
        self.label_51.setPixmap(QtGui.QPixmap(":/icon/icons/M_Favicon.ico"))
        self.label_51.setScaledContents(True)
        self.label_51.setObjectName("label_51")
        self.gridLayout.addWidget(self.label_51, 1, 0, 1, 1)
        self.label_28 = QtWidgets.QLabel(self.widget_2)
        self.label_28.setMinimumSize(QtCore.QSize(0, 25))
        self.label_28.setObjectName("label_28")
        self.gridLayout.addWidget(self.label_28, 4, 5, 1, 1)
        self.label_45 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_45.setFont(font)
        self.label_45.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_45.setAlignment(QtCore.Qt.AlignCenter)
        self.label_45.setObjectName("label_45")
        self.gridLayout.addWidget(self.label_45, 10, 7, 1, 1)
        self.label_35 = QtWidgets.QLabel(self.widget_2)
        self.label_35.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_35.setFont(font)
        self.label_35.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_35.setAlignment(QtCore.Qt.AlignCenter)
        self.label_35.setObjectName("label_35")
        self.gridLayout.addWidget(self.label_35, 3, 7, 1, 1)
        self.label_25 = QtWidgets.QLabel(self.widget_2)
        self.label_25.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_25.setFont(font)
        self.label_25.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_25.setAlignment(QtCore.Qt.AlignCenter)
        self.label_25.setObjectName("label_25")
        self.gridLayout.addWidget(self.label_25, 2, 7, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.widget_2)
        self.label_5.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 5, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.widget_2)
        self.label_11.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.gridLayout.addWidget(self.label_11, 7, 5, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.widget_2)
        self.label_9.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 6, 5, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.widget_2)
        self.label_21.setMinimumSize(QtCore.QSize(0, 25))
        self.label_21.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_21.setFont(font)
        self.label_21.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_21.setAlignment(QtCore.Qt.AlignCenter)
        self.label_21.setObjectName("label_21")
        self.gridLayout.addWidget(self.label_21, 7, 7, 1, 1)
        self.label_30 = QtWidgets.QLabel(self.widget_2)
        self.label_30.setMinimumSize(QtCore.QSize(0, 25))
        self.label_30.setObjectName("label_30")
        self.gridLayout.addWidget(self.label_30, 8, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.widget_2)
        self.label_10.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 5, 5, 1, 1)
        self.label_33 = QtWidgets.QLabel(self.widget_2)
        self.label_33.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_33.setFont(font)
        self.label_33.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_33.setAlignment(QtCore.Qt.AlignCenter)
        self.label_33.setObjectName("label_33")
        self.gridLayout.addWidget(self.label_33, 8, 7, 1, 1)
        self.label_64 = QtWidgets.QLabel(self.widget_2)
        self.label_64.setMaximumSize(QtCore.QSize(25, 25))
        self.label_64.setText("")
        self.label_64.setPixmap(QtGui.QPixmap(":/icon/icons/校验_check-one.svg"))
        self.label_64.setScaledContents(True)
        self.label_64.setObjectName("label_64")
        self.gridLayout.addWidget(self.label_64, 2, 4, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.widget_2)
        self.label_2.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 1, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.widget_2)
        self.label_16.setMinimumSize(QtCore.QSize(0, 25))
        self.label_16.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_16.setAlignment(QtCore.Qt.AlignCenter)
        self.label_16.setObjectName("label_16")
        self.gridLayout.addWidget(self.label_16, 7, 2, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.widget_2)
        self.label_15.setMinimumSize(QtCore.QSize(0, 25))
        self.label_15.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_15.setFont(font)
        self.label_15.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_15.setAlignment(QtCore.Qt.AlignCenter)
        self.label_15.setObjectName("label_15")
        self.gridLayout.addWidget(self.label_15, 6, 2, 1, 1)
        self.label_29 = QtWidgets.QLabel(self.widget_2)
        self.label_29.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_29.setFont(font)
        self.label_29.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_29.setAlignment(QtCore.Qt.AlignCenter)
        self.label_29.setObjectName("label_29")
        self.gridLayout.addWidget(self.label_29, 4, 7, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.widget_2)
        self.label_18.setMinimumSize(QtCore.QSize(0, 25))
        self.label_18.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_18.setAlignment(QtCore.Qt.AlignCenter)
        self.label_18.setObjectName("label_18")
        self.gridLayout.addWidget(self.label_18, 1, 7, 1, 1)
        self.label_27 = QtWidgets.QLabel(self.widget_2)
        self.label_27.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_27.setFont(font)
        self.label_27.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_27.setAlignment(QtCore.Qt.AlignCenter)
        self.label_27.setObjectName("label_27")
        self.gridLayout.addWidget(self.label_27, 4, 2, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.widget_2)
        self.label_14.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.gridLayout.addWidget(self.label_14, 7, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.widget_2)
        self.label_12.setMinimumSize(QtCore.QSize(0, 25))
        self.label_12.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_12.setFont(font)
        self.label_12.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_12.setAlignment(QtCore.Qt.AlignCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout.addWidget(self.label_12, 1, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.widget_2)
        self.label_6.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 5, 1, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.widget_2)
        self.label_24.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_24.setFont(font)
        self.label_24.setObjectName("label_24")
        self.gridLayout.addWidget(self.label_24, 2, 5, 1, 1)
        self.label_42 = QtWidgets.QLabel(self.widget_2)
        self.label_42.setObjectName("label_42")
        self.gridLayout.addWidget(self.label_42, 10, 1, 1, 1)
        self.label_38 = QtWidgets.QLabel(self.widget_2)
        self.label_38.setObjectName("label_38")
        self.gridLayout.addWidget(self.label_38, 9, 1, 1, 1)
        self.label_36 = QtWidgets.QLabel(self.widget_2)
        self.label_36.setObjectName("label_36")
        self.gridLayout.addWidget(self.label_36, 3, 1, 1, 1)
        self.label_43 = QtWidgets.QLabel(self.widget_2)
        self.label_43.setObjectName("label_43")
        self.gridLayout.addWidget(self.label_43, 10, 5, 1, 1)
        self.label_39 = QtWidgets.QLabel(self.widget_2)
        self.label_39.setObjectName("label_39")
        self.gridLayout.addWidget(self.label_39, 9, 5, 1, 1)
        self.label_34 = QtWidgets.QLabel(self.widget_2)
        self.label_34.setMinimumSize(QtCore.QSize(0, 25))
        self.label_34.setObjectName("label_34")
        self.gridLayout.addWidget(self.label_34, 3, 5, 1, 1)
        self.label_41 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_41.setFont(font)
        self.label_41.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_41.setAlignment(QtCore.Qt.AlignCenter)
        self.label_41.setObjectName("label_41")
        self.gridLayout.addWidget(self.label_41, 9, 7, 1, 1)
        self.label_44 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_44.setFont(font)
        self.label_44.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_44.setAlignment(QtCore.Qt.AlignCenter)
        self.label_44.setObjectName("label_44")
        self.gridLayout.addWidget(self.label_44, 10, 2, 1, 1)
        self.label_46 = QtWidgets.QLabel(self.widget_2)
        self.label_46.setObjectName("label_46")
        self.gridLayout.addWidget(self.label_46, 11, 1, 1, 1)
        self.label_47 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_47.setFont(font)
        self.label_47.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_47.setAlignment(QtCore.Qt.AlignCenter)
        self.label_47.setObjectName("label_47")
        self.gridLayout.addWidget(self.label_47, 11, 2, 1, 1)
        self.label_48 = QtWidgets.QLabel(self.widget_2)
        self.label_48.setObjectName("label_48")
        self.gridLayout.addWidget(self.label_48, 11, 5, 1, 1)
        self.label_49 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_49.setFont(font)
        self.label_49.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_49.setAlignment(QtCore.Qt.AlignCenter)
        self.label_49.setObjectName("label_49")
        self.gridLayout.addWidget(self.label_49, 11, 7, 1, 1)
        self.label_37 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_37.setFont(font)
        self.label_37.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_37.setAlignment(QtCore.Qt.AlignCenter)
        self.label_37.setObjectName("label_37")
        self.gridLayout.addWidget(self.label_37, 3, 2, 1, 1)
        self.label_32 = QtWidgets.QLabel(self.widget_2)
        self.label_32.setMinimumSize(QtCore.QSize(0, 25))
        self.label_32.setObjectName("label_32")
        self.gridLayout.addWidget(self.label_32, 8, 5, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.widget_2)
        self.label_8.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 6, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.widget_2)
        self.label_17.setMinimumSize(QtCore.QSize(0, 25))
        self.label_17.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_17.setAlignment(QtCore.Qt.AlignCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout.addWidget(self.label_17, 0, 7, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.widget_2)
        self.label_20.setMinimumSize(QtCore.QSize(0, 25))
        self.label_20.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_20.setAlignment(QtCore.Qt.AlignCenter)
        self.label_20.setObjectName("label_20")
        self.gridLayout.addWidget(self.label_20, 6, 7, 1, 1)
        self.label_26 = QtWidgets.QLabel(self.widget_2)
        self.label_26.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_26.setFont(font)
        self.label_26.setObjectName("label_26")
        self.gridLayout.addWidget(self.label_26, 4, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.widget_2)
        self.label_3.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 5, 1, 1)
        self.label_22 = QtWidgets.QLabel(self.widget_2)
        self.label_22.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.gridLayout.addWidget(self.label_22, 2, 1, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.widget_2)
        self.label_19.setMinimumSize(QtCore.QSize(0, 25))
        self.label_19.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_19.setFont(font)
        self.label_19.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_19.setAlignment(QtCore.Qt.AlignCenter)
        self.label_19.setObjectName("label_19")
        self.gridLayout.addWidget(self.label_19, 5, 7, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.widget_2)
        self.label_7.setMinimumSize(QtCore.QSize(0, 25))
        self.label_7.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_7.setFont(font)
        self.label_7.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 0, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.widget_2)
        self.label_4.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 1, 1, 1)
        self.label_40 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_40.setFont(font)
        self.label_40.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_40.setAlignment(QtCore.Qt.AlignCenter)
        self.label_40.setObjectName("label_40")
        self.gridLayout.addWidget(self.label_40, 9, 2, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.widget_2)
        self.label_13.setMinimumSize(QtCore.QSize(0, 25))
        self.label_13.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_13.setFont(font)
        self.label_13.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_13.setAlignment(QtCore.Qt.AlignCenter)
        self.label_13.setObjectName("label_13")
        self.gridLayout.addWidget(self.label_13, 5, 2, 1, 1)
        self.label_31 = QtWidgets.QLabel(self.widget_2)
        self.label_31.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_31.setFont(font)
        self.label_31.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_31.setAlignment(QtCore.Qt.AlignCenter)
        self.label_31.setObjectName("label_31")
        self.gridLayout.addWidget(self.label_31, 8, 2, 1, 1)
        self.label_23 = QtWidgets.QLabel(self.widget_2)
        self.label_23.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_23.setAlignment(QtCore.Qt.AlignCenter)
        self.label_23.setObjectName("label_23")
        self.gridLayout.addWidget(self.label_23, 2, 2, 1, 1)
        self.label_50 = QtWidgets.QLabel(self.widget_2)
        self.label_50.setMaximumSize(QtCore.QSize(25, 25))
        self.label_50.setText("")
        self.label_50.setPixmap(QtGui.QPixmap(":/icon/icons/上一步_back.svg"))
        self.label_50.setScaledContents(True)
        self.label_50.setObjectName("label_50")
        self.gridLayout.addWidget(self.label_50, 0, 0, 1, 1)
        self.label_55 = QtWidgets.QLabel(self.widget_2)
        self.label_55.setMaximumSize(QtCore.QSize(25, 25))
        self.label_55.setText("")
        self.label_55.setPixmap(QtGui.QPixmap(":/icon/icons/复制_copy.svg"))
        self.label_55.setScaledContents(True)
        self.label_55.setObjectName("label_55")
        self.gridLayout.addWidget(self.label_55, 3, 0, 1, 1)
        self.label_70 = QtWidgets.QLabel(self.widget_2)
        self.label_70.setMaximumSize(QtCore.QSize(25, 25))
        self.label_70.setText("")
        self.label_70.setPixmap(QtGui.QPixmap(":/icon/icons/下一步_next.svg"))
        self.label_70.setScaledContents(True)
        self.label_70.setObjectName("label_70")
        self.gridLayout.addWidget(self.label_70, 8, 4, 1, 1)
        self.label_63 = QtWidgets.QLabel(self.widget_2)
        self.label_63.setMaximumSize(QtCore.QSize(25, 25))
        self.label_63.setText("")
        self.label_63.setPixmap(QtGui.QPixmap(":/icon/icons/截屏_screenshot-two.svg"))
        self.label_63.setScaledContents(True)
        self.label_63.setObjectName("label_63")
        self.gridLayout.addWidget(self.label_63, 11, 0, 1, 1)
        self.label_62 = QtWidgets.QLabel(self.widget_2)
        self.label_62.setMaximumSize(QtCore.QSize(25, 25))
        self.label_62.setText("")
        self.label_62.setPixmap(QtGui.QPixmap(":/icon/icons/相交选择_intersect-selection.svg"))
        self.label_62.setScaledContents(True)
        self.label_62.setObjectName("label_62")
        self.gridLayout.addWidget(self.label_62, 10, 0, 1, 1)
        self.label_73 = QtWidgets.QLabel(self.widget_2)
        self.label_73.setMaximumSize(QtCore.QSize(25, 25))
        self.label_73.setText("")
        self.label_73.setPixmap(QtGui.QPixmap(":/icon/icons/截屏_screenshot-two.svg"))
        self.label_73.setScaledContents(True)
        self.label_73.setObjectName("label_73")
        self.gridLayout.addWidget(self.label_73, 11, 4, 1, 1)
        self.label_59 = QtWidgets.QLabel(self.widget_2)
        self.label_59.setMaximumSize(QtCore.QSize(25, 25))
        self.label_59.setText("")
        self.label_59.setPixmap(QtGui.QPixmap(":/icon/icons/照片_pic.svg"))
        self.label_59.setScaledContents(True)
        self.label_59.setObjectName("label_59")
        self.gridLayout.addWidget(self.label_59, 7, 0, 1, 1)
        self.label_72 = QtWidgets.QLabel(self.widget_2)
        self.label_72.setMaximumSize(QtCore.QSize(25, 25))
        self.label_72.setText("")
        self.label_72.setPixmap(QtGui.QPixmap(":/icon/icons/排除选择_exclude-selection.svg"))
        self.label_72.setScaledContents(True)
        self.label_72.setObjectName("label_72")
        self.gridLayout.addWidget(self.label_72, 10, 4, 1, 1)
        self.label_58 = QtWidgets.QLabel(self.widget_2)
        self.label_58.setMaximumSize(QtCore.QSize(25, 25))
        self.label_58.setText("")
        self.label_58.setPixmap(QtGui.QPixmap(":/icon/icons/去顶部_to-top.svg"))
        self.label_58.setScaledContents(True)
        self.label_58.setObjectName("label_58")
        self.gridLayout.addWidget(self.label_58, 6, 0, 1, 1)
        self.label_61 = QtWidgets.QLabel(self.widget_2)
        self.label_61.setMaximumSize(QtCore.QSize(25, 25))
        self.label_61.setText("")
        self.label_61.setPixmap(QtGui.QPixmap(":/icon/icons/合并选择_union-selection.svg"))
        self.label_61.setScaledContents(True)
        self.label_61.setObjectName("label_61")
        self.gridLayout.addWidget(self.label_61, 9, 0, 1, 1)
        self.label_54 = QtWidgets.QLabel(self.widget_2)
        self.label_54.setMaximumSize(QtCore.QSize(25, 25))
        self.label_54.setText("")
        self.label_54.setPixmap(QtGui.QPixmap(":/icon/icons/删除_delete-two.svg"))
        self.label_54.setScaledContents(True)
        self.label_54.setObjectName("label_54")
        self.gridLayout.addWidget(self.label_54, 2, 0, 1, 1)
        self.label_53 = QtWidgets.QLabel(self.widget_2)
        self.label_53.setMaximumSize(QtCore.QSize(25, 25))
        self.label_53.setText("")
        self.label_53.setPixmap(QtGui.QPixmap(":/icon/icons/锚点_anchor.svg"))
        self.label_53.setScaledContents(True)
        self.label_53.setObjectName("label_53")
        self.gridLayout.addWidget(self.label_53, 1, 4, 1, 1)
        self.label_52 = QtWidgets.QLabel(self.widget_2)
        self.label_52.setMaximumSize(QtCore.QSize(25, 25))
        self.label_52.setText("")
        self.label_52.setPixmap(QtGui.QPixmap(":/icon/icons/下一步_next.svg"))
        self.label_52.setScaledContents(True)
        self.label_52.setObjectName("label_52")
        self.gridLayout.addWidget(self.label_52, 0, 4, 1, 1)
        self.label_56 = QtWidgets.QLabel(self.widget_2)
        self.label_56.setMaximumSize(QtCore.QSize(25, 25))
        self.label_56.setText("")
        self.label_56.setPixmap(QtGui.QPixmap(":/icon/icons/关闭_close-one.svg"))
        self.label_56.setScaledContents(True)
        self.label_56.setObjectName("label_56")
        self.gridLayout.addWidget(self.label_56, 4, 0, 1, 1)
        self.label_57 = QtWidgets.QLabel(self.widget_2)
        self.label_57.setMaximumSize(QtCore.QSize(25, 25))
        self.label_57.setText("")
        self.label_57.setPixmap(QtGui.QPixmap(":/icon/icons/删除_delete.svg"))
        self.label_57.setScaledContents(True)
        self.label_57.setObjectName("label_57")
        self.gridLayout.addWidget(self.label_57, 5, 0, 1, 1)
        self.label_60 = QtWidgets.QLabel(self.widget_2)
        self.label_60.setMaximumSize(QtCore.QSize(25, 25))
        self.label_60.setText("")
        self.label_60.setPixmap(QtGui.QPixmap(":/icon/icons/上一步_back.svg"))
        self.label_60.setScaledContents(True)
        self.label_60.setObjectName("label_60")
        self.gridLayout.addWidget(self.label_60, 8, 0, 1, 1)
        self.label_71 = QtWidgets.QLabel(self.widget_2)
        self.label_71.setMaximumSize(QtCore.QSize(25, 25))
        self.label_71.setText("")
        self.label_71.setPixmap(QtGui.QPixmap(":/icon/icons/减去上一层_subtract-selection-one.svg"))
        self.label_71.setScaledContents(True)
        self.label_71.setObjectName("label_71")
        self.gridLayout.addWidget(self.label_71, 9, 4, 1, 1)
        self.label_68 = QtWidgets.QLabel(self.widget_2)
        self.label_68.setMaximumSize(QtCore.QSize(25, 25))
        self.label_68.setText("")
        self.label_68.setPixmap(QtGui.QPixmap(":/icon/icons/去底部_to-bottom.svg"))
        self.label_68.setScaledContents(True)
        self.label_68.setObjectName("label_68")
        self.gridLayout.addWidget(self.label_68, 6, 4, 1, 1)
        self.label_66 = QtWidgets.QLabel(self.widget_2)
        self.label_66.setMaximumSize(QtCore.QSize(25, 25))
        self.label_66.setText("")
        self.label_66.setPixmap(QtGui.QPixmap(":/icon/icons/眼睛_eyes.svg"))
        self.label_66.setScaledContents(True)
        self.label_66.setObjectName("label_66")
        self.gridLayout.addWidget(self.label_66, 4, 4, 1, 1)
        self.label_65 = QtWidgets.QLabel(self.widget_2)
        self.label_65.setMaximumSize(QtCore.QSize(25, 25))
        self.label_65.setText("")
        self.label_65.setPixmap(QtGui.QPixmap(":/icon/icons/编辑撰写_writing-fluently.svg"))
        self.label_65.setScaledContents(True)
        self.label_65.setObjectName("label_65")
        self.gridLayout.addWidget(self.label_65, 3, 4, 1, 1)
        self.label_67 = QtWidgets.QLabel(self.widget_2)
        self.label_67.setMaximumSize(QtCore.QSize(25, 25))
        self.label_67.setText("")
        self.label_67.setPixmap(QtGui.QPixmap(":/icon/icons/保存_save.svg"))
        self.label_67.setScaledContents(True)
        self.label_67.setObjectName("label_67")
        self.gridLayout.addWidget(self.label_67, 5, 4, 1, 1)
        self.label_69 = QtWidgets.QLabel(self.widget_2)
        self.label_69.setMaximumSize(QtCore.QSize(25, 25))
        self.label_69.setText("")
        self.label_69.setPixmap(QtGui.QPixmap(":/icon/icons/全宽_fullwidth.svg"))
        self.label_69.setScaledContents(True)
        self.label_69.setObjectName("label_69")
        self.gridLayout.addWidget(self.label_69, 7, 4, 1, 1)
        self.label_74 = QtWidgets.QLabel(self.widget_2)
        self.label_74.setMaximumSize(QtCore.QSize(25, 16777215))
        self.label_74.setText("")
        self.label_74.setObjectName("label_74")
        self.gridLayout.addWidget(self.label_74, 0, 3, 1, 1)
        self.verticalLayout.addWidget(self.widget_2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Shortcut"))
        self.label.setText(_translate("Dialog", "Shortcut"))
        self.label_28.setText(_translate("Dialog", "Polygons Visible"))
        self.label_45.setText(_translate("Dialog", "Ctrl + E"))
        self.label_35.setText(_translate("Dialog", "R"))
        self.label_25.setText(_translate("Dialog", "E"))
        self.label_5.setText(_translate("Dialog", "Draw polygon"))
        self.label_11.setText(_translate("Dialog", "Zoom fit"))
        self.label_9.setText(_translate("Dialog", "To bottom"))
        self.label_21.setText(_translate("Dialog", "F"))
        self.label_30.setText(_translate("Dialog", "Prev group"))
        self.label_10.setText(_translate("Dialog", "Save annotation"))
        self.label_33.setText(_translate("Dialog", "Tab"))
        self.label_2.setText(_translate("Dialog", "Prev image"))
        self.label_16.setText(_translate("Dialog", "Space"))
        self.label_15.setText(_translate("Dialog", "T"))
        self.label_29.setText(_translate("Dialog", "V"))
        self.label_18.setText(_translate("Dialog", "C"))
        self.label_27.setText(_translate("Dialog", "Esc"))
        self.label_14.setText(_translate("Dialog", "Bit map"))
        self.label_12.setText(_translate("Dialog", "Q"))
        self.label_6.setText(_translate("Dialog", "Delete polygon"))
        self.label_24.setText(_translate("Dialog", "Annotate finish"))
        self.label_42.setText(_translate("Dialog", "Polygon intersect"))
        self.label_38.setText(_translate("Dialog", "Polygon union"))
        self.label_36.setText(_translate("Dialog", "Copy"))
        self.label_43.setText(_translate("Dialog", "Polygon exclude"))
        self.label_39.setText(_translate("Dialog", "Polygon subtract"))
        self.label_34.setText(_translate("Dialog", "Repaint"))
        self.label_41.setText(_translate("Dialog", "Ctrl + D"))
        self.label_44.setText(_translate("Dialog", "Ctrl + Q"))
        self.label_46.setText(_translate("Dialog", "Scene shot"))
        self.label_47.setText(_translate("Dialog", "P"))
        self.label_48.setText(_translate("Dialog", "Window shot"))
        self.label_49.setText(_translate("Dialog", "Ctrl + P"))
        self.label_37.setText(_translate("Dialog", "Ctrl + C"))
        self.label_32.setText(_translate("Dialog", "Next group"))
        self.label_8.setText(_translate("Dialog", "To top"))
        self.label_17.setText(_translate("Dialog", "D"))
        self.label_20.setText(_translate("Dialog", "B"))
        self.label_26.setText(_translate("Dialog", "Annotate cancel"))
        self.label_3.setText(_translate("Dialog", "Next image"))
        self.label_22.setText(_translate("Dialog", "Backspace"))
        self.label_19.setText(_translate("Dialog", "S"))
        self.label_7.setText(_translate("Dialog", "A"))
        self.label_4.setText(_translate("Dialog", "Segment anything"))
        self.label_40.setText(_translate("Dialog", "Ctrl + A"))
        self.label_13.setText(_translate("Dialog", "Del"))
        self.label_31.setText(_translate("Dialog", "`"))
        self.label_23.setText(_translate("Dialog", "Z"))

