# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'anno_dock.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(339, 302)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Form)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.checkBox_visible = QtWidgets.QCheckBox(self.widget)
        self.checkBox_visible.setMaximumSize(QtCore.QSize(120, 16777215))
        self.checkBox_visible.setObjectName("checkBox_visible")
        self.horizontalLayout.addWidget(self.checkBox_visible)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.comboBox_group_select = QtWidgets.QComboBox(self.widget)
        self.comboBox_group_select.setMaximumSize(QtCore.QSize(60, 16777215))
        self.comboBox_group_select.setObjectName("comboBox_group_select")
        self.horizontalLayout.addWidget(self.comboBox_group_select)
        self.button_prev_group = QtWidgets.QPushButton(self.widget)
        self.button_prev_group.setMaximumSize(QtCore.QSize(25, 16777215))
        self.button_prev_group.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icon/icons/上一步_back.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_prev_group.setIcon(icon)
        self.button_prev_group.setObjectName("button_prev_group")
        self.horizontalLayout.addWidget(self.button_prev_group)
        self.button_next_group = QtWidgets.QPushButton(self.widget)
        self.button_next_group.setMaximumSize(QtCore.QSize(25, 16777215))
        self.button_next_group.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icon/icons/下一步_next.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.button_next_group.setIcon(icon1)
        self.button_next_group.setObjectName("button_next_group")
        self.horizontalLayout.addWidget(self.button_next_group)
        self.label = QtWidgets.QLabel(self.widget)
        self.label.setMinimumSize(QtCore.QSize(3, 0))
        self.label.setMaximumSize(QtCore.QSize(3, 16777215))
        self.label.setText("")
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout.addWidget(self.widget)
        self.listWidget = QtWidgets.QListWidget(Form)
        self.listWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.checkBox_visible.setText(_translate("Form", "Visible"))
        self.comboBox_group_select.setStatusTip(_translate("Form", "Select polygons by group."))
        self.button_prev_group.setStatusTip(_translate("Form", "Prev group."))
        self.button_next_group.setStatusTip(_translate("Form", "Next group."))

