# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'about_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(550, 200)
        Dialog.setMinimumSize(QtCore.QSize(0, 200))
        Dialog.setMaximumSize(QtCore.QSize(800, 300))
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(10, 10, 10, 10)
        self.horizontalLayout.setSpacing(10)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.widget)
        self.label.setMinimumSize(QtCore.QSize(180, 180))
        self.label.setMaximumSize(QtCore.QSize(180, 180))
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/icon/icons/ISAT11_100.svg"))
        self.label.setScaledContents(True)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.widget_3 = QtWidgets.QWidget(self.widget)
        self.widget_3.setObjectName("widget_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget_3)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.widget_3)
        self.label_3.setMaximumSize(QtCore.QSize(16777215, 40))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(20)
        self.label_3.setFont(font)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.label_version = QtWidgets.QLabel(self.widget_3)
        self.label_version.setMinimumSize(QtCore.QSize(0, 25))
        self.label_version.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_version.setFont(font)
        self.label_version.setStatusTip("")
        self.label_version.setWhatsThis("")
        self.label_version.setAccessibleName("")
        self.label_version.setAccessibleDescription("")
        self.label_version.setAutoFillBackground(False)
        self.label_version.setText("V0.0.1")
        self.label_version.setAlignment(QtCore.Qt.AlignCenter)
        self.label_version.setOpenExternalLinks(False)
        self.label_version.setObjectName("label_version")
        self.verticalLayout_2.addWidget(self.label_version)
        self.label_5 = QtWidgets.QLabel(self.widget_3)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_5.setFont(font)
        self.label_5.setOpenExternalLinks(True)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_2.addWidget(self.label_5)
        self.horizontalLayout.addWidget(self.widget_3)
        self.verticalLayout.addWidget(self.widget)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "about"))
        self.label_3.setText(_translate("Dialog", "ISAT with Segment anything"))
        self.label_5.setText(_translate("Dialog", "<html><head/><body><p align=\"center\">Open source on <a href=\"https://github.com/yatengLG/ISAT_with_segment_anything\"><span style=\" font-weight:600; text-decoration: underline; color:#0000ff;\">Github</span></a>.</p><p align=\"center\">Main contributor <a href=\"https://github.com/Alias-z\"><span style=\" font-weight:600; text-decoration: underline; color:#0000ff;\">Alias-z</span></a>.</p><p align=\"center\">ISAT Copyright (C) 2022 <a href=\"https://github.com/yatengLG\"><span style=\" text-decoration: underline; color:#0000ff;\">yatengLG</span></a>.</p></body></html>"))

